/////////////////////////////////////////////////////////////////////////////
///
/// @file m_differ.h
///
/// A sample difference equation.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2014 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started on April 10, 2008. Last revision: April 10, 2008.


#ifndef _CMGRAPHS_DIFFERENCE_H_
#define _CMGRAPHS_DIFFERENCE_H_

// the generic class for the type of a map
#include "maptype.h"

// the interval arithmetic data types
#include "typeintv.h"


// --------------------------------------------------
// -------- a sample difference equation map --------
// --------------------------------------------------

/// This class defines a map derived from a sample difference equation.
/// There are only two parameters of this map: 'b' and 'h'.
class MapDifference: public MapType
{
public:
	/// The default constructor.
	MapDifference ();

	/// Computes the image of a box whose left and right coordinates
	/// are given. Fills in the images with the left and right
	/// coordinates of the image box.
	void compute (const double *xleft, const double *xright,
		double *yleft, double *yright, int dim,
		const spcCoord *coord, int subdiv) const;

}; /* class MapDifference */

// --------------------------------------------------

inline MapDifference::MapDifference ()
{
	testIntervals ();
	return;
} /* MapDifference::MapDifference */

inline void MapDifference::compute (const double *xleft, const double *xright,
	double *yleft, double *yright, int dim, const spcCoord *, int) const
{
	IntervalType b (getLeftParam (0), getRightParam (0));
	IntervalType h (getLeftParam (1), getRightParam (1));
	for (int i = 0; i < dim - 1; ++ i)
	{
		yleft [i] = xleft [i + 1];
		yright [i] = xright [i + 1];
	}

	IntervalType x (xleft [0], xright [0]);
	IntervalType y (xleft [1], xright [1]);
	IntervalType result = (b * y) / (1 + x) - h;
	double left = result. leftBound ();
	yleft [dim - 1] = (left <= 0) ? 0 : left;
	double right = result. rightBound ();
	yright [dim - 1] = (right <= 0) ? 0 : right;

	// swich the rounding direction to the neutral one
	resetRounding ();
	return;
} /* MapDifference::compute */


#endif // _CMGRAPHS_DIFFERENCE_H_

