/////////////////////////////////////////////////////////////////////////////
///
/// @file m_vpol.h
///
/// The time-t map for the Vanderpol equations.
/// This map defines a sample time-t map for an ODE.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2014 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started on October 20, 2008. Last revision: February 11, 2012.


#ifndef _CMGRAPHS_VANDERPL_H_
#define _CMGRAPHS_VANDERPL_H_


// include local header files
#include "maptype.h"
#include "odetimet.h"


// --------------------------------------------------
// ------------ the Vanderpol time-t map ------------
// --------------------------------------------------

/// This class defines the time-t map for the Vanderpol equations.
class MapVanderPol: public MapOdeTimeT
{
public:
	/// The constructor of a Vanderpol map object.
	MapVanderPol ();

}; /* class MapVanderPol */

// --------------------------------------------------

inline MapVanderPol::MapVanderPol ():
#ifdef REVERSED_TIME
// [Vanderpol equations with reversed time:]
	MapOdeTimeT ("par:pa;var:x,y;"
		"fun:-y+(x*((x*x)-pa)),x;", 3, 0.001, 10)
#else
// [Vanderpol equations:]
	MapOdeTimeT ("par:pa;var:x,y;"
		"fun:y-(x*((x*x)-pa)),-x;", 3, 0.001, 10)
#endif
{
	return;
} /* MapVanderPol::MapVanderPol */


#endif // _CMGRAPHS_VANDERPL_H_

