#!/usr/bin/perl

#############################################################################
#
# This Perl script adds up the times indicated in the execution logs
# created by the Conley-Morse graphs software.
#
#############################################################################

# Copyright (C) 1997-2014 by Pawel Pilarczyk.
#
# This file is part of my research software package. This is free software:
# you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file "license.txt". If not,
# please, see <https://www.gnu.org/licenses/>.

# Started on June 25, 2007. Last revision: December 5, 2010.

use Date::Parse;

die "Call with log file names to sum the times from." unless scalar @ARGV;

sub showtime
{
	my $title = shift;
	my $sum = shift;
	my $sum_m = int ($sum * 10 / 6) / 100.0;
	my $sum_h = int ($sum / 36) / 100.0;
	print "$title: $sum sec = $sum_m min = $sum_h hours.\n";
}

sub dateTime2epoch
{
#	my @months = ("Jan","Feb","Mar","Apr","May","Jun",
#		"Jul","Aug","Sep","Oct","Nov","Dec");
}

my $unknown = 0;
my $filecount = 0;
my $maxtime = 0;
my $sum = 0.0;
foreach my $pattern (@ARGV)
{
	foreach my $filename (glob $pattern)
	{
		next unless -f $filename;
		open ($file, $filename) or next;
		my $fileTime = (stat($filename))[9];
	#	print "DEBUG: file time = ${fileTime}\n";
		my $startTime;
		my $time = 0.0;
		my $found = 0;
		++ $filecount;
		while (<$file>)
		{
			if (m/^Start time: (\S+ \S+ \d+ \d\d:\d\d:\d\d \d\d\d\d)/)
			{
				$startTime = str2time ($1);
			#	print "DEBUG: start time = ${startTime}\n";
			}
			if (m/^Results for .* \(processed in ([\d+\.]+) sec\) accepted/)
			{
				$time += $1;
				++ $found;
			}
			if (m/^Total time used\: ([\d+\.]+) sec /)
			{
				$time += $1;
				++ $found;
			}
		}
		my $deducedTime = defined ($startTime) ?
			($fileTime - $startTime) : 0;
		my $deducedInfo = defined ($startTime) ?
			"; deduced time: ${deducedTime}" : "";
		print "$filename - $time seconds${deducedInfo}.\n"
			if ($found and $time);
		print "$filename - time not found${deducedInfo}.\n"
			unless $found;
		$time = $deducedTime if (!$found && defined ($startTime));
		$maxtime = $time if ($maxtime < $time);
		++ $unknown unless $found;
		$sum += $time;
		close $file;
	}
}

print "= = = = = = = = = = = = = = = = = = = =\n";

print "${filecount} files processed.\n";

print "Warning: ${unknown} times are not known (interrupted computatons?).\n"
	if $unknown;

showtime ("Total time (single CPU)", $sum);
#showtime ("Total time (42 cluster)", int ($sum / 42));

print "Total cost at 0.011 euro per CPU-hour: ",
	int ($sum / 3600 * 0.011), " euro.\n";

